\name{subsetting-utils}

\alias{subsetting-utils}

\alias{class:NSBS}
\alias{NSBS-class}

\alias{length,NSBS-method}
\alias{anyDuplicated.NSBS}
\alias{anyDuplicated,NSBS-method}
\alias{isStrictlySorted,NSBS-method}
\alias{max,NSBS-method}

\alias{NSBS}
\alias{NSBS,NSBS-method}
\alias{NSBS,ANY-method}

\alias{as.integer,NativeNSBS-method}

\alias{as.integer,RangeNSBS-method}
\alias{length,RangeNSBS-method}
\alias{anyDuplicated,RangeNSBS-method}
\alias{isStrictlySorted,RangeNSBS-method}
\alias{max,RangeNSBS-method}
\alias{show,RangeNSBS-method}

\alias{NSBS,missing-method}
\alias{NSBS,NULL-method}
\alias{NSBS,numeric-method}
\alias{NSBS,logical-method}
\alias{NSBS,character-method}
\alias{NSBS,factor-method}

\alias{normalizeSingleBracketSubscript}
\alias{normalizeSingleBracketReplacementValue}
\alias{normalizeSingleBracketReplacementValue,ANY-method}

\alias{extractROWS}
\alias{extractROWS,ANY,ANY-method}
\alias{extractROWS,vector_OR_factor,RangeNSBS-method}
\alias{extractROWS,array,RangeNSBS-method}
\alias{extractROWS,data.frame,RangeNSBS-method}
\alias{replaceROWS}
\alias{replaceROWS,ANY,ANY-method}
\alias{mergeROWS}
\alias{mergeROWS,ANY,ANY-method}
\alias{extractCOLS}
\alias{replaceCOLS}

\alias{normalizeDoubleBracketSubscript}

\alias{getListElement}
\alias{setListElement}
\alias{getListElement,list-method}
\alias{setListElement,list-method}


\title{Subsetting utilities}

\description{
  Low-level utility functions and classes defined in the \pkg{S4Vectors}
  package to support subsetting of vector-like objects. They are not
  intended to be used directly.
}

\keyword{utilities}
\keyword{classes}
\keyword{methods}
